/*
 * EERAM_7_segment.C
 * Cycle a 7-Segment display with power failure mitigation
 * using the 47x04/47x16 EERAM to save last value
 *********************************************************
 **** This software is offered strictly as is with no ****
 **** warranties whatsoever. Use it at your own risk.** **
 *********************************************************
 */
#include "clk.h"
#include "qm_common.h"
#include "qm_gpio.h"
#include "qm_i2c.h"
#include "qm_pinmux.h"
#include "qm_pin_functions.h"

/* function prototypes */
int setI2C(void);
int readbyteC(uint16_t address);
int writebyteC(uint16_t address,uint8_t byte);
void display_digit(uint8_t digit);
void display_error(void);
void mismatch(void);

#define NO 0
#define YES 1

/* note if connecting the HS pin to GPIO_3 uncomment the following line */
/* #define HS_PIN (QM_PIN_ID_3) */

/* these are the 7 segments a-g */
#define Sa_PIN (QM_PIN_ID_8)
#define Sb_PIN (QM_PIN_ID_9)
#define Sc_PIN (QM_PIN_ID_10)
#define Sd_PIN (QM_PIN_ID_11)
#define Se_PIN (QM_PIN_ID_16)
#define Sf_PIN (QM_PIN_ID_17)
#define Sg_PIN (QM_PIN_ID_18)

/* EERAM I2C address is defined as uint16_t because the master read prototype wants them that way.
 * We could have used a uint8_t which logically makes sense but would be converted to unit16_t.
 * Note that these addresses assume A0=0 and A1=0, change to match your connections.
 */
const uint16_t SRAMcb=0x50;  /* address of 47x04/47x16 control byte for SRAM with A0=0 and A1=0 */
                             /* change if necessary                                             */

/* globals */
/*----------------------------------------------------------------------------------*/
/* NOTE: User should set or check the next three variables                          */
bool Hdensity=NO;       /* chip density, either NO/0=47x04 or YES/1=47x16           */
/* These are the EERAM SRAM locations (addresses) to save the count for the display */
uint16_t LOC1pre=200;
uint16_t LOC2post=202;
/*----------------------------------------------------------------------------------*/

qm_gpio_port_config_t GPIOcfg;  /* GPIO configuration structure- see qm_gpio.h */
qm_i2c_config_t I2Ccfg; 	    /* I2C structure variables                     */
qm_i2c_status_t I2Cstatus; 	    /* see i2c.h for values                        */

/* array to set the pin to segment - *must be in order (segments A-G) and correspond to the GPIOs* */
/*int pin_seg[7]={8,9,10,11,16,17,18};*/
int pin_seg[7]={QM_PIN_ID_8,QM_PIN_ID_9,QM_PIN_ID_10,QM_PIN_ID_11,QM_PIN_ID_16,QM_PIN_ID_17,QM_PIN_ID_18};
/* arrays to set the segments for digits  */
uint8_t num_array[10][7] = {
		{ 1,1,1,1,1,1,0 },    /* 0 */
        { 0,1,1,0,0,0,0 },    /* 1 */
        { 1,1,0,1,1,0,1 },    /* 2 */
        { 1,1,1,1,0,0,1 },    /* 3 */
        { 0,1,1,0,0,1,1 },    /* 4 */
        { 1,0,1,1,0,1,1 },    /* 5 */
        { 1,0,1,1,1,1,1 },    /* 6 */
        { 1,1,1,0,0,0,0 },    /* 7 */
        { 1,1,1,1,1,1,1 },    /* 8 */
        { 1,1,1,0,0,1,1 }};   /* 9 */
/*___________________________________________________________________________*/
int main(void)
{
	int rescode;
	int n,o;

	/* delay to complete a power-on restore */
	if(Hdensity){
			clk_sys_udelay(5000); /* 5 milliseconds delay */
		}
		else{
			clk_sys_udelay(2000); /* 2 milliseconds delay */
		}
	/* configure GPIO bits - the mux defaults to mode 0 but let's be sure */
	/* HS bit */
	/* note if connecting the HS pin to GPIO_3 uncomments the following two lines */
	/* qm_pmux_select(HS_PIN, QM_PIN_0_FN_GPIO_0); */
	/* GPIOcfg.direction |= BIT(HS_PIN); */

	/* 7 segments */
	qm_pmux_select(Sa_PIN, QM_PIN_8_FN_GPIO_8);
	GPIOcfg.direction |= BIT(Sa_PIN);
	qm_pmux_select(Sb_PIN, QM_PIN_9_FN_GPIO_9);
	GPIOcfg.direction |= BIT(Sb_PIN);
	qm_pmux_select(Sb_PIN, QM_PIN_10_FN_GPIO_10);
	GPIOcfg.direction |= BIT(Sc_PIN);
	qm_pmux_select(Sc_PIN, QM_PIN_10_FN_GPIO_10);
	GPIOcfg.direction |= BIT(Sc_PIN);
	qm_pmux_select(Sd_PIN, QM_PIN_11_FN_GPIO_11);
	GPIOcfg.direction |= BIT(Sd_PIN);
	qm_pmux_select(Se_PIN, QM_PIN_16_FN_GPIO_16);
	GPIOcfg.direction |= BIT(Se_PIN);
	qm_pmux_select(Sf_PIN, QM_PIN_17_FN_GPIO_17);
	GPIOcfg.direction |= BIT(Sf_PIN);
	qm_pmux_select(Sg_PIN, QM_PIN_18_FN_GPIO_18);
	GPIOcfg.direction |= BIT(Sg_PIN);
	qm_gpio_set_config(QM_GPIO_0, &GPIOcfg);

	/* note if connecting the HS pin to GPIO_3 uncomment the following line */
	/* qm_gpio_clear_pin(QM_GPIO_0, HS_PIN); */ /* to be safe */
	/* The HS_PIN is used to trigger auto-store when AM bit=1.This feature is not used here	*/

	/* configure I2C and quit if error - no I2C, no program */
	if ( (rescode=setI2C() )!=0 ) {
		return 1;
	}
	/* Here we assume we have I/O and I2C ready to go, so read SRAM LOC1pre and LOC2post as ints to catch errors.
	 * Establish the display on power up *without* updating and with the normal delay - customize as needed. We
	 * also check for errors but we do not rewrite LOC1pre and LOC2post except when establishing the initial values.
	 */

	/* read LOC1pre */
	if( (n=readbyteC(LOC1pre) )<0){
		/* we have a terminal error denoted by a negative value that can be printed out for debugging */
		display_error();  /* flash 'E' */
	}
	/* read LOC2post */
	if( (o=readbyteC(LOC2post) )<0){
		/* we have a terminal error denoted by a negative value that can be printed out for debugging */
		display_error();  /* flash 'E' */
	}
	/* LOC1pre is in 'n' LOC2post is in 'o' */
	/* Check for *very* unlikely case of mismatched LOC1pre and LOC2post and process if desired. */
	if(n!=o){
		mismatch();
	}
	/* check for n>9 terminal error (should never occur except on initial run with start-up value of 255) */
	if(n>9){
		if(n==255){
			/* if n=255 assume that this is a fresh start so initialize to 0 */
			n=0;
			/* write initial LOC1pre */
			if((writebyteC(LOC1pre,(uint8_t)n)!=0) ) {
				display_error(); /* terminal error if bad write */
			}
			/* write initial LOC2post */
			o=n;
			if((writebyteC(LOC2post,(uint8_t)o)!=0) ) {
				display_error(); /* terminal error if bad write */
			}
		}
		else{
			/* n>9 and not 255 which is an illegal value */
			display_error();  /* flash 'E' */
		}
	}
	/* we have a valid n and LOC1pre and LOC2post, display the digit, delay and enter the main loop */
	display_digit((uint8_t)n);
	clk_sys_udelay(3000000L); /* delay for 3 seconds */

	while(1){ /* loop forever - just update and display */
		/* read LOC1pre */
		if( (n=readbyteC(LOC1pre) )<0){
			/* we have a terminal error denoted by a negative value that can be printed out for debugging */
			display_error();  /* flash 'E' */
		}
		/* read LOC2post */
		if( (o=readbyteC(LOC2post) )<0){
			/* we have a terminal error denoted by a negative value that can be printed out for debugging */
			display_error();  /* flash 'E' */
		}
		/* LOC1pre is in 'n' LOC2post is in 'o' */
		/* update LOC1pre */
		n++;
		if(n==10){
			n=0;
		}
		if((writebyteC(LOC1pre,(uint8_t)n)!=0) ) {
			display_error(); /* terminal error if bad write */
		}
		display_digit((uint8_t)n);
		/* update LOC2post */
		o=n;
		if((writebyteC(LOC2post,(uint8_t)o)!=0) ) {
			display_error(); /* terminal error if bad write */
		}
		clk_sys_udelay(3000000L); /* delay for 3 seconds */
	}
	return 0;  /* never gets here */
}
/*__________________________________________________________________________*/
int setI2C(void) {

	/*  Enable I2C 0 */
	clk_periph_enable(CLK_PERIPH_CLK | CLK_PERIPH_I2C_M0_REGISTER);
	/* set IO pins for SDA and SCL */
	qm_pmux_select(QM_PIN_ID_6, QM_PMUX_FN_2);
	qm_pmux_select(QM_PIN_ID_7, QM_PMUX_FN_2);

	/* Configure I2C */
	I2Ccfg.address_mode = QM_I2C_7_BIT;
	I2Ccfg.mode = QM_I2C_MASTER;
	I2Ccfg.speed = QM_I2C_SPEED_STD;

	/* set the configuration through the structure and return if failure */
	if (qm_i2c_set_config(QM_I2C_0, &I2Ccfg)) {
		QM_PUTS("Error: I2C_0 config\n");
		return (errno);
	} else {
		return (0);
	}
}
/*___________________________________________________________________________*/
int readbyteC(uint16_t address)
{
	/* read EERAM SRAM byte at address with error checking
	 * returns byte (as an int) if read without error
	 * returns -999 if address out of bounds based on the Hdensity variable
	 * returns negated errno if I2C failure
	 */
	int rescode;
	uint16_t length, maxaddr;
	uint8_t addset[2],R_array[1];

	if(Hdensity){
		maxaddr=2047;
	}
	else{
		maxaddr=511;
	}
	/* check for address out of bounds */
	if(address>maxaddr){
		return(-999);
	}
	/* set the address to the SRAM address with a write (no data)	 */
	addset[0]=(uint8_t)address >> 8; /* SRAM address high byte */
	addset[1]=(uint8_t)address&0xff; /* SRAM address low byte  */
	/* write the array address with NO stop bit */
	length=2;
	rescode=qm_i2c_master_write(QM_I2C_0, SRAMcb, addset, length, false, &I2Cstatus);
	if(rescode) {
	    return((rescode*=-1)); /* ERROR: negate errno and return */
		}
	/* now read the SRAM byte into E_array */
	length=1;
	rescode=qm_i2c_master_read(QM_I2C_0, SRAMcb, R_array, length, true, &I2Cstatus);
	if(rescode) {
		return((rescode*=-1)); /* ERROR: negate errno and return */
	}
	else{
		return(R_array[0]);   /* return byte read */
	}
}
/*___________________________________________________________________________*/

int writebyteC(uint16_t address,uint8_t byte)
{
	/* write EERAM SRAM byte at address with error checking
	 * returns 0 (as an int) if written without error
	 * returns -999 if address out of bounds based on the Hdensity variable
	 * returns errno if I2C failure
	 */
	int rescode;
	uint16_t length, maxaddr;
	uint8_t addset[3];

	if(Hdensity){
		maxaddr=2047;
	}
	else{
		maxaddr=511;
	}
	/* check for address out of bounds */
	if(address>maxaddr){
		return(-999);
	}
	/* set the address to the SRAM address 	*/
	addset[0]=(uint8_t)address >> 8;        /* SRAM address high byte */
	addset[1]=(uint8_t)address&0xff;        /* SRAM address low byte  */
	addset[2]=byte;                         /* put the byte to write into addset[2]  */
	/* write byte to SRAM array address    */
	length=3;
	rescode=qm_i2c_master_write(QM_I2C_0, SRAMcb, addset, length, true, &I2Cstatus);
	return(rescode); /* 0 if ok else errno */
}
/*___________________________________________________________________________*/
void display_digit(uint8_t digit){
	/*
	 * NOTE:  QM_PIN_ID_0 - QM_PIN_ID_24 are enum vars in qm_pinmux.h
	 * we sequentially define the GPIOs we use in pin_seg[7]
	 * If you use a different set of GPIOs you will need to define them
	 * sequentially in pin_seg[7].
	 */
	if(digit<10){ /* only allow 0-9 */
		for (int x=0; x < 7; x++) {
			if(num_array[digit][x]==1)
			{
				qm_gpio_set_pin(QM_GPIO_0,pin_seg[x]);
			}
			else
			{
				qm_gpio_clear_pin(QM_GPIO_0,(pin_seg[x]));
			}
		}
	}
}
/*___________________________________________________________________________*/
void display_error(void){
	/*
	 * Flash the letter 'E' on the 7-segment display to denote a terminal
	 * error. Endless loop indicated user intervention is needed.
	 */
	uint8_t edigit[7]={ 1,0,0,1,1,1,1 };

	while(1){  /* <-- NOTE endless loop */
		for (int x=0; x < 7; x++) {
			if(edigit[x]==1)
			{
				qm_gpio_set_pin(QM_GPIO_0,pin_seg[x]);
			}
			else
			{
				qm_gpio_clear_pin(QM_GPIO_0,(pin_seg[x]));
			}
		}
		clk_sys_udelay(1000000L);  /* 1 sec */
		for (int x=0; x < 7; x++) {
			qm_gpio_clear_pin(QM_GPIO_0,(pin_seg[x]));
		}
		clk_sys_udelay(500000L);  /* .5 sec */
	}
}
/*___________________________________________________________________________*/
void mismatch(void){
	/* called in the *very* unlikely case that LOC1pre!=LOC2post
	 * could occur if power failed after updating LOC1_pre and before
	 * updating LOC2post. The main loop handles this by assuming LOC1pre
	 * is the correct value and acts accordingly. User can provide additional
	 * processing here if possible. As it is, no action is taken.
	 */
}
/*_______________________________END__________________________________________*/


