/*
 * EERAM_CMD_SR.C
 * Software store and recall commands to transfer
 * EERAM SRAM to and from EPROM on the 47x04/47x16
 *********************************************************
 **** This software is offered strictly as is with no ****
 **** warranties whatsoever. Use it at your own risk.** **
 *********************************************************
 */
#include "clk.h"
#include "qm_common.h"
#include "qm_gpio.h"
#include "qm_i2c.h"
#include "qm_pinmux.h"
#include "qm_pin_functions.h"

#define NO 0
#define YES 1

/* NOTE: GPIO 3 can be connected to EERAM HS pin (change as needed)
 * but is not used in this program  uncomment the next line if used */
/* #define HS_PIN (QM_PIN_ID_3) */

/* prototypes */
int setI2C(void);
int readbyteC(uint16_t address);
int writebyteC(uint16_t address,uint8_t byte);
int writeCMD(uint8_t CMD);

/* EERAM addresses are defined as uint16_t because the master read prototype wants them that way.
 * We could have used a uint8_t which logically makes sense but would be converted to unit16_t.
 * Note that these addresses assume A0=0 and A1=0, change to match your connections.
 */
const uint16_t EERAMcr=0x18;    // address of 47x04/47x16 control register with A0=0 and A1=0
const uint16_t SRAMcb=0x50;     // address of 47x04/47x16 control byte forSRAM with A0=0 and A1=0

/* these are not I2C device addresses but are sent after the device address */
const uint8_t COMMANDreg=0x55;  /* address of 47x04/47x16 command register  */
const uint8_t STATUSreg=0x00;   /* address of 47x04/47x16 status register   */
/* these are the two command register commands */
const uint8_t S_store=0x33;
const uint8_t S_recall=0xdd;

/* globals */

/* ---> NOTE: User should set the next variable <--- */
bool Hdensity=NO;     /* chip density, either NO/0=47x04 or YES/1=47x16 */
/*-----*/

/* read and write arrays for I2C, these are passed as a constant pointer to non-constant data */
uint8_t Rarray[1];
uint8_t Warray[2];

qm_gpio_port_config_t GPIOcfg;  /* GPIO configuration structure- see qm_gpio.h */
qm_i2c_config_t I2Ccfg; 	    /* I2C structure variables                     */
qm_i2c_status_t I2Cstatus; 	    /* see i2c.h for values                        */
/*___________________________________________________________________________*/
int main(void)
{
	uint8_t readByte, writeByte;
	uint16_t address;
	int rescode;

	QM_PRINTF("EERAM SRAM Software save and restore\n");
	/* configure GPIO bit for the HS connection and clear it */
	/* uncomment the next four lines if using the HS line    */
	/* qm_pmux_select(HS_PIN, QM_PIN_0_FN_GPIO_0); */
	/* GPIOcfg.direction = BIT(HS_PIN);            */
	/* qm_gpio_set_config(QM_GPIO_0, &GPIOcfg);    */
	/* qm_gpio_clear_pin(QM_GPIO_0, HS_PIN);       */
	/* The HS_PIN is used to trigger auto-store when AM bit=1.This feature is not used here	*/

	/* configure I2C and quit if error */
	if ( (rescode=setI2C() )!=0 ) {
		QM_PRINTF("\nError: Could not configure I2C (errno=%d)\n",rescode);
		return 1;
	}

	/* read byte at address 0x20 */
	address=0x020;
	if( (rescode=readbyteC(address) )<0 ){
		if((rescode=-999)){
			QM_PRINTF("\n ERROR: Address Out of Range");
		}
		else{
			QM_PRINTF("\n ERROR: %d", (rescode*-1));
		}
	}
	else{
		readByte=(uint8_t)rescode;
		QM_PRINTF("\n SRAM Byte at address %d=%d", address,readByte);
	}
	/* issue a software save of the SRAM array to EPROM */
	QM_PRINTF("\n Saving SRAM to EPROM");
	if ( (rescode=writeCMD(S_store) !=0) ){
		QM_PRINTF("\n ERROR: %d",rescode);
	}
	else{
		QM_PRINTF("\n Saved OK");
	}
	/* change the value and rewrite the byte to address */
	writeByte= readByte + 0x10;
	if( (rescode=writebyteC(address,writeByte) )!=0 ){
		if((rescode==-999)){
			QM_PRINTF("\n ERROR: Address Out of Range");
		}
		else{
			QM_PRINTF("\n ERROR: %d",rescode);
		}
	}
	else{
		QM_PRINTF("\n Wrote %d to SRAM Byte at address %d",writeByte, address);
	}

	/* read byte at address again to verify the write */
	if( (rescode=readbyteC(address) )<0 ){
		if((rescode=-999)){
			QM_PRINTF("\n ERROR: Address Out of Range");
		}
		else{
			QM_PRINTF("\n ERROR: %d", (rescode*-1));
		}
	}
	else{
		readByte=(uint8_t)rescode;
		QM_PRINTF("\n SRAM Byte at address %d=%d", address,readByte);
	}
	/* now issue a software recall of the EPROM to SRAM */
	QM_PRINTF("\n Recalling EPROM to SRAM");
	if ( (rescode=writeCMD(S_recall) !=0) ){
		QM_PRINTF("\n ERROR: %d",rescode);
	}
	else{
		QM_PRINTF("\n Restore OK");
	}

	/* read byte at address again to verify the restore */
	address=0x020;
	if( (rescode=readbyteC(address) )<0 ){
		if((rescode=-999)){
			QM_PRINTF("\n ERROR: Address Out of Range");
		}
		else{
			QM_PRINTF("\n ERROR: %d", (rescode*-1));
		}
	}
	else{
		readByte=(uint8_t)rescode;
		QM_PRINTF("\n SRAM Byte at address %d=%d", address,readByte);
	}
	return 0;
}
/*___________________________________________________________________________*/
int setI2C(void) {

	/*  Enable I2C 0 */
	clk_periph_enable(CLK_PERIPH_CLK | CLK_PERIPH_I2C_M0_REGISTER);
	/* set IO pins for SDA and SCL */
	qm_pmux_select(QM_PIN_ID_6, QM_PMUX_FN_2);
	qm_pmux_select(QM_PIN_ID_7, QM_PMUX_FN_2);

	/* Configure I2C */
	I2Ccfg.address_mode = QM_I2C_7_BIT;
	I2Ccfg.mode = QM_I2C_MASTER;
	I2Ccfg.speed = QM_I2C_SPEED_STD;

	/* set the configuration through the structure and return if failure */
	if (qm_i2c_set_config(QM_I2C_0, &I2Ccfg)) {
		QM_PUTS("Error: I2C_0 config\n");
		return (errno);
	} else {
		return (0);
	}
}
/*___________________________________________________________________________*/

int readbyteC(uint16_t address){
	/* read EERAM SRAM byte at address with error checking
	 * returns byte (as an int) if read without error
	 * returns -999 if address out of bounds based on the Hdensity variable
	 * returns negated errno if I2C failure
	 */
	int rescode;
	uint16_t length, maxaddr;
	uint8_t addset[2],R_array[1];

	if(Hdensity){
		maxaddr=2047;
	}
	else{
		maxaddr=511;
	}
	/* check for address out of bounds */
	if(address>maxaddr){
		return(-999);
	}
	/* set the address to the SRAM address with a write (no data)	 */
	addset[0]=(uint8_t)address >> 8; /* SRAM address high byte */
	addset[1]=(uint8_t)address&0xff; /* SRAM address low byte  */
	/* write the array address with NO stop bit */
	length=2;
	rescode=qm_i2c_master_write(QM_I2C_0, SRAMcb, addset, length, false, &I2Cstatus);
	if(rescode) {
		return((rescode*=-1)); /* ERROR: negate errno and return */
	}
	/* now read the SRAM byte into E_array */
	length=1;
	rescode=qm_i2c_master_read(QM_I2C_0, SRAMcb, R_array, length, true, &I2Cstatus);
	if(rescode) {
		return((rescode*=-1)); /* ERROR: negate errno and return */
	}
	else{
		return(R_array[0]);   /* return byte read */
	}
}
/*___________________________________________________________________________*/

int writebyteC(uint16_t address,uint8_t byte)
{
	/* write EERAM SRAM byte at address with error checking
	 * returns 0 (as an int) if written without error
	 * returns -999 if address out of bounds based on the Hdensity variable
	 * returns errno if I2C failure
	 */
	int rescode;
	uint16_t length, maxaddr;
	uint8_t addset[3];

	if(Hdensity){
		maxaddr=2047;
	}
	else{
		maxaddr=511;
	}
	/* check for address out of bounds */
	if(address>maxaddr){
		return(-999);
	}
	/* set the address to the SRAM address 	*/
	addset[0]=(uint8_t)address >> 8;        /* SRAM address high byte */
	addset[1]=(uint8_t)address&0xff;        /* SRAM address low byte  */
	addset[2]=byte;                         /* put the byte to write into addset[2]  */
	/* write byte to SRAM array address    */
	length=3;
	rescode=qm_i2c_master_write(QM_I2C_0, SRAMcb, addset, length, true, &I2Cstatus);
	return(rescode); /* 0 if ok else errno */
}
/*___________________________________________________________________________*/
int writeCMD(uint8_t CMD)
{
	/* write command to the command register with error checking
	 * returns 0 (as an int) if written without error
	 * returns errno if I2C failure
	 */

	int length=2;
	int rescode;
	uint8_t cmdset[2];

	cmdset[0]=COMMANDreg;
	cmdset[1]=CMD;
	rescode=qm_i2c_master_write(QM_I2C_0, EERAMcr, cmdset, length, true, &I2Cstatus);
	if(rescode==0){
		/* do the delays */
		if(CMD==S_store){
			if(Hdensity){
				clk_sys_udelay(25000); /* 25 milliseconds delay */
			}
			else{
				clk_sys_udelay(8000); /* 8 milliseconds delay */
			}
		}
		else if(CMD==S_recall){
			if(Hdensity){
				clk_sys_udelay(5000); /* 5 milliseconds delay */
			}
			else{
				clk_sys_udelay(2000); /* 2 milliseconds delay */
			}
		}
	}
	return(rescode); /* 0 if ok else errno */
}
/*___________________________________________________________________________*/
/* end */



