/*
 * EERAM_CNTRL_STATUS_REG.C
 * Read and write to the 47x04/47x16 EERAM Status register
 *********************************************************
 **** This software is offered strictly as is with no ****
 **** warranties whatsoever. Use it at your own risk.** **
 *********************************************************
 */

#include "clk.h"
#include "qm_common.h"
#include "qm_gpio.h"
#include "qm_i2c.h"
#include "qm_pinmux.h"
#include "qm_pin_functions.h"

#define NO 0
#define YES 1

/* NOTE: uncomment next line if using GPIO 3 connected to EERAM HS pin (change as needed) */
/* #define HS_PIN (QM_PIN_ID_3) */

/* prototypes */
int setI2C(void);
void print_binary(uint8_t byte);
void print_regbits(uint8_t byte);

/* EERAM addresses are defined as uint16_t because the master read prototype wants them that way.
 * We could have used a uint8_t which logically makes sense but would be converted to unit16_t.
 * Note that these addresses assume A0=0 and A1=0, change to match your connections.
 */
const uint16_t EERAMcr=0x18;    /* address of 47x04/47x16 control register with A0=0 and A1=0 */

/* these are not I2C device addresses but are sent after the device address */
const uint8_t COMMANDreg=0x55;  /* address of 47x04/47x16 command register  */
const uint8_t STATUSreg=0x00;   /* address of 47x04/47x16 status register   */

/* globals */

/*-----*/
/* ---> NOTE: User should set the next THREE variables <--- */
bool Hdensity=NO;                   /* chip density, either NO/0=47x04 or YES/1=47x16 */
bool WriteStatus=NO;                /* If YES, write newval to STATUS register if NO skip the write  */
/* uint8_t newval=0b00000010; no blocks protected, enable auto store, clear EVENT bit if set */
uint8_t newval=0b00000000;          /* no blocks protected, disable auto store, clear EVENT bit if set */
/*
 * newval reflects a new STATUS register value
 * b0 - EVENT bit, NON-VOLATILE - set to 1 when HS pin is driven high
 * b1 - ASE Auto-store bit 1=enabled 0=disabled
 * b2-b4 are block protect bits, see data sheet
 * b2 - BP0
 * b3 - BP1
 * b4 - BP2
 * b5 - unused
 * b6 - unused
 * b7 - AM bit, READ ONLY/VOLATILE - Array Modified
 */
/*-----*/

/* read and write arrays for I2C, these are passed as a constant pointer to non-constant data */
uint8_t Rarray[1];
uint8_t Warray[2];

qm_gpio_port_config_t GPIOcfg;  /* GPIO configuration structure- see qm_gpio.h */
qm_i2c_config_t I2Ccfg; 	    /* I2C structure variables                     */
qm_i2c_status_t I2Cstatus; 	    /* see i2c.h for values                        */

/*___________________________________________________________________________*/

int main(void) {

	int rescode;
	uint16_t length;

	QM_PRINTF("EERAM Configuration Register Explorer\n\n");

	/* configure GPIO bit for the HS connection and clear it              */
	/* The HS_PIN is used to trigger auto-store when AM bit=1.            */
	/* This feature is not used here - uncomment next four lines if using */
	/* qm_pmux_select(HS_PIN, QM_PIN_0_FN_GPIO_0); */
	/* GPIOcfg.direction = BIT(HS_PIN);            */
	/* qm_gpio_set_config(QM_GPIO_0, &GPIOcfg);    */
	/* qm_gpio_clear_pin(QM_GPIO_0, HS_PIN);       */

	/* configure I2C and quit if error */
	if ( (rescode=setI2C() ) ) {
		QM_PRINTF("Error: Could not configure I2C (errno=%d)\n",rescode);
		return 1;
	}

	length=1;
	/* read the control register (reads to the control register always read the status register) */
	rescode=qm_i2c_master_read(QM_I2C_0, EERAMcr, Rarray, length, true, &I2Cstatus);
	if(rescode) {
		QM_PRINTF("Error reading EERAM STATUS register (errno=%d)\n",rescode);
		return 1;
	}
	QM_PRINTF("EERAM STATUS register = ");
	print_binary(Rarray[0]);
	/* print interpretation of status reg bits */
	print_regbits(Rarray[0]);

	if(WriteStatus){
		Warray[0]=STATUSreg;
		Warray[1]=newval;
		length=2;
		QM_PRINTF("\nwriting STATUS register...\n");
		rescode=qm_i2c_master_write(QM_I2C_0, EERAMcr, Warray, length, true, &I2Cstatus);
		clk_sys_udelay(1000); /* Twc 1 millisecond delay */
		if(rescode) {
			QM_PRINTF(" Error writing EERAM STATUS register (errno=%d)\n",rescode);
			return 1;
		}
		else{
			QM_PRINTF("\nEERAM STATUS register = ");
			length=1;
			/* read the control register (reads to the control register always read the status register) */
			rescode=qm_i2c_master_read(QM_I2C_0, EERAMcr, Rarray, length, true, &I2Cstatus);
			if(rescode) {
				QM_PRINTF("Error reading EERAM STATUS register (errno=%d)\n",rescode);
				return 1;
			}
			print_binary(Rarray[0]);
			/* print post-write interpretation of status reg bits */
			print_regbits(Rarray[0]);
		}
	}
	return 0;
}

/*__________________________________________________________________________*/
int setI2C(void) {

	/*  Enable I2C 0 */
	clk_periph_enable(CLK_PERIPH_CLK | CLK_PERIPH_I2C_M0_REGISTER);
	/* set IO pins for SDA and SCL */
	qm_pmux_select(QM_PIN_ID_6, QM_PMUX_FN_2);
	qm_pmux_select(QM_PIN_ID_7, QM_PMUX_FN_2);

	/* Configure I2C */
	I2Ccfg.address_mode = QM_I2C_7_BIT;
	I2Ccfg.mode = QM_I2C_MASTER;
	I2Ccfg.speed = QM_I2C_SPEED_STD;

	/* set the configuration through the structure and return if failure */
	if (qm_i2c_set_config(QM_I2C_0, &I2Ccfg)) {
		QM_PUTS("Error: I2C_0 config\n");
		return (errno);
	} else {
		return (0);
	}
}
/*___________________________________________________________________________*/

void print_binary(uint8_t byte){
	/* print byte in binary */
	int x;
	for(x=0; x<8; x++)
	{
		if(byte & 0x80)
		{
			QM_PRINTF(" 1");
		}
		else
		{
			QM_PRINTF(" 0");
		}
		byte = byte<<1;
	}
}
/*___________________________________________________________________________*/

void print_regbits(uint8_t byte){
	/* print out the interpretation of the status register bits (from LSB to MSB) */
	uint8_t bpb;
	QM_PRINTF("\n");
	/* Bit 0- EVENT bit */
	if(byte&1){
		QM_PRINTF("Event detected on HS pin\n");
	}
	else{
		QM_PRINTF("No event detected on HS pin\n");
	}
	/* Bit 1- ASE (auto-store enable) bit */
	if(byte&2){
		QM_PRINTF("Auto-Store is enabled\n");
	}
	else{
		QM_PRINTF("Auto-Store is disabled\n");
	}

	/* Bits 2,3,4 - Block Protect bits */
	bpb = (byte & 0b0011100) >> 2;
	switch (bpb) {
	case 0:
		QM_PRINTF("No protected array addresses\n");
		break;
	case 1:
		QM_PRINTF("Upper 1/64 protected (addresses ");
		if (Hdensity){
			QM_PRINTF("0x7E0-0x7FF)\n");
		}
		else{
			QM_PRINTF("0x1F8-0x1FF)\n");
		}
		break;
	case 2:
		QM_PRINTF("Upper 1/32 protected (addresses ");
		if (Hdensity){
			QM_PRINTF("0x7C0-0x7FF)\n");
		}
		else{
			QM_PRINTF("0x1F0-0x1FF)\n");
		}
		break;
	case 3:
		QM_PRINTF("Upper 1/16 protected (addresses ");
		if(Hdensity){
			QM_PRINTF("0x780-0x7FF)\n");
		}
		else{
			QM_PRINTF("0x1E0-0x1FF)\n");
		}
		break;
	case 4:
		QM_PRINTF("Upper 1/8 protected (addresses ");
		if(Hdensity){
			QM_PRINTF("0x700-0x7FF)\n");
		}
		else{
			QM_PRINTF("0x1C0-0x1FF)\n");
		}
		break;
	case 5:
		QM_PRINTF("Upper 1/4 protected (addresses ");
		if(Hdensity){
			QM_PRINTF("0x600-0x7FF)\n");
		}
		else{
			QM_PRINTF("0x180-0x1FF)\n");
		}
		break;
	case 6:
		QM_PRINTF("Upper 1/2 protected (addresses ");
		if(Hdensity){
			QM_PRINTF("0x400-0x7FF)\n");
		}
		else{
			QM_PRINTF("0x100-0x1FF)\n");
		}
		break;
	case 7:
		QM_PRINTF("All blocks protected (addresses ");
		if(Hdensity){
			QM_PRINTF("0x000-0x7FF)\n");
		}
		else{
			QM_PRINTF("0x000-0x1FF)\n");
		}
		break;
	default:
		/* serious problem if you get here */
		QM_PRINTF("\nUnspecified read error\n");
		break;
	}
	/* note: Bits 5-6 are not used */
	/* Bit 7=AM (array modified) bit */
	if(byte&0x80){
		QM_PRINTF("SRAM array has been modified\n");
	}
	else{
		QM_PRINTF("SRAM array has not been modified\n");
	}

}
/* end */



