/*
 * EERAM_RW.C
 * Read / Write SRAM on the 47x04/47x16 EERAM
 *********************************************************
 **** This software is offered strictly as is with no ****
 **** warranties whatsoever. Use it at your own risk. ****
 *********************************************************
 */
#include "clk.h"
#include "qm_common.h"
#include "qm_gpio.h"
#include "qm_i2c.h"
#include "qm_pinmux.h"
#include "qm_pin_functions.h"

#define NO 0
#define YES 1

/* NOTE: uncomment next line if using GPIO 3 connected to EERAM HS pin (change as needed) */
/* #define HS_PIN (QM_PIN_ID_3) */

/* prototypes */
int setI2C(void);
int readbyteC(uint16_t address);
int writebyteC(uint16_t address,uint8_t byte);

/* EERAM I2C address is defined as uint16_t because the master read prototype wants them that way.
 * We could have used a uint8_t which logically makes sense but would be converted to unit16_t.
 * Note that these addresses assume A0=0 and A1=0, change to match your connections.
 */
const uint16_t SRAMcb=0x50;     // address of 47x04/47x16 control byte for SRAM with A0=0 and A1=0

/* globals */
/*-----*/
/* ---> NOTE: User should set this variable <---                          */
bool Hdensity=NO;       /* chip density, either NO/0=47x04 or YES/1=47x16 */
/*-----*/

qm_gpio_port_config_t GPIOcfg;  /* GPIO configuration structure- see qm_gpio.h */
qm_i2c_config_t I2Ccfg; 	    /* I2C structure variables                     */
qm_i2c_status_t I2Cstatus; 	    /* see i2c.h for values                        */

/*__________________________________________________________________________*/

int main(void)
{
	uint8_t readByte, writeByte;
	uint16_t address;
	int rescode;

	QM_PRINTF("EERAM SRAM Read / Write bytes\n");

	/* configure GPIO bit for the HS connection and clear it              */
    /* The HS_PIN is used to trigger auto-store when AM bit=1.            */
	/* This feature is not used here - uncomment next four lines if using */
	/* qm_pmux_select(HS_PIN, QM_PIN_0_FN_GPIO_0); */
	/* GPIOcfg.direction = BIT(HS_PIN);            */
	/* qm_gpio_set_config(QM_GPIO_0, &GPIOcfg);    */
	/* qm_gpio_clear_pin(QM_GPIO_0, HS_PIN);       */
	/* The HS_PIN is used to trigger auto-store when AM bit=1.This feature is not used here	*/

	/* configure I2C and quit if error */
	if ( (rescode=setI2C() )!=0 ) {
		QM_PRINTF("\nError: Could not configure I2C (errno=%d)\n",rescode);
		return 1;
	}

	/* read byte at address 0x0f */
	address=0x0f;
    if( (rescode=readbyteC(address) )<0 ){
    	if((rescode=-999)){
    	   QM_PRINTF("\n ERROR: Address Out of Range");
    	}
    	else{
    	   QM_PRINTF("\n ERROR: %d", (rescode*-1));
    	}
    }
    else{
    	readByte=(uint8_t)rescode;
    	QM_PRINTF("\n SRAM Byte at address %d=%d", address,readByte);
    }

   /* write 0b01010101 to address */
    writeByte=0b01010101;
    if( (rescode=writebyteC(address,writeByte) )!=0 ){
    	if((rescode==-999)){
    	   QM_PRINTF("\n ERROR: Address Out of Range");
    	}
    	else{
    	   QM_PRINTF("\n ERROR: %d",rescode);
    	}
    }
    else{
    	QM_PRINTF("\n Wrote %d to SRAM Byte at address %d",writeByte, address);
    }

	/* read byte at address 500 again to verify the write */
	address=0x0f;
    if( (rescode=readbyteC(address) )<0 ){
    	if((rescode=-999)){
    	   QM_PRINTF("\n ERROR: Address Out of Range");
    	}
    	else{
    	   QM_PRINTF("\n ERROR: %d", (rescode*-1));
    	}
    }
    else{
    	readByte=(uint8_t)rescode;
    	QM_PRINTF("\n SRAM Byte at address %d=%d", address,readByte);
    }


	return 0;
}
/*___________________________________________________________________________*/
int setI2C(void) {

	/*  Enable I2C 0 */
	clk_periph_enable(CLK_PERIPH_CLK | CLK_PERIPH_I2C_M0_REGISTER);
	/* set IO pins for SDA and SCL */
	qm_pmux_select(QM_PIN_ID_6, QM_PMUX_FN_2);
	qm_pmux_select(QM_PIN_ID_7, QM_PMUX_FN_2);

	/* Configure I2C */
	I2Ccfg.address_mode = QM_I2C_7_BIT;
	I2Ccfg.mode = QM_I2C_MASTER;
	I2Ccfg.speed = QM_I2C_SPEED_STD;

	/* set the configuration through the structure and return if failure */
	if (qm_i2c_set_config(QM_I2C_0, &I2Ccfg)) {
		QM_PUTS("Error: I2C_0 config\n");
		return (errno);
	} else {
		return (0);
	}
}
/*___________________________________________________________________________*/

int readbyteC(uint16_t address){
	/* read EERAM SRAM byte at address with error checking
	 * returns byte (as an int) if read without error
	 * returns -999 if address out of bounds based on the Hdensity variable
	 * returns negated errno if I2C failure
	 */

	int rescode;
	uint16_t length, maxaddr;
	uint8_t addset[2],R_array[1];

	if(Hdensity){
		maxaddr=2047;
	}
	else{
		maxaddr=511;
	}
	/* check for address out of bounds */
	if(address>maxaddr){
		return(-999);
	}
	/* set the address to the SRAM address with a write (no data)	 */
	addset[0]=(uint8_t)address >> 8; /* SRAM address high byte */
	addset[1]=(uint8_t)address&0xff; /* SRAM address low byte  */
	/* write the array address with NO stop bit */
	length=2;
	rescode=qm_i2c_master_write(QM_I2C_0, SRAMcb, addset, length, false, &I2Cstatus);
	if(rescode) {
	    return((rescode*=-1)); /* ERROR: negate errno and return */
		}
	/* now read the SRAM byte into E_array */
	length=1;
	rescode=qm_i2c_master_read(QM_I2C_0, SRAMcb, R_array, length, true, &I2Cstatus);
	if(rescode) {
		return((rescode*=-1)); /* ERROR: negate errno and return */
	}
	else{
		return(R_array[0]);   /* return byte read */
	}

}
/*___________________________________________________________________________*/

int writebyteC(uint16_t address,uint8_t byte)
{
	/* write EERAM SRAM byte at address with error checking
	 * returns 0 (as an int) if written without error
	 * returns -999 if address out of bounds based on the Hdensity variable
	 * returns errno if I2C failure
	 */

	int rescode;
	uint16_t length, maxaddr;
	uint8_t addset[3];

	if(Hdensity){
		maxaddr=2047;
	}
	else{
		maxaddr=511;
	}
	/* check for address out of bounds */
	if(address>maxaddr){
		return(-999);
	}
	/* set the address to the SRAM address 	*/
	addset[0]=(uint8_t)address >> 8;        /* SRAM address high byte */
	addset[1]=(uint8_t)address&0xff;        /* SRAM address low byte  */
	addset[2]=byte;                         /* put the byte to write into addset[2]  */
	/* write byte to SRAM array address    */
	length=3;
	rescode=qm_i2c_master_write(QM_I2C_0, SRAMcb, addset, length, true, &I2Cstatus);
	return(rescode); /* 0 if ok else errno */

}
/*___________________________________________________________________________*/
/* end */
