/*
 * EERAM_S_RW.C
 * Read/Write all SRAM on the 47x04/47x16 EERAM
 *********************************************************
 **** This software is offered strictly as is with no ****
 **** warranties whatsoever. Use it at your own risk. ****
 *********************************************************
 */
#include "clk.h"
#include "qm_common.h"
#include "qm_gpio.h"
#include "qm_i2c.h"
#include "qm_pinmux.h"
#include "qm_pin_functions.h"

#define NO 0
#define YES 1
/* NOTE: uncomment next line if using GPIO 3 connected to EERAM HS pin (change as needed) */
/* #define HS_PIN (QM_PIN_ID_3) */

/* prototypes */
int setI2C(void);
int write_SRAM(void);
int read_SRAM();

/* EERAM addresses are defined as uint16_t because the master read prototype wants them that way.
 * We could have used a uint8_t which logically makes sense but would be converted to unit16_t.
 * Note that these addresses assume A0=0 and A1=0, change to match your connections.
 */
const uint16_t SRAMcb=0x50;     // address of 47x04/47x16 control byte for SRAM with A0=0 and A1=0

/* globals */
/*-----*/
/* ---> NOTE: User should set these two variables <---                    */
bool Hdensity=NO;       /* chip density, either NO/0=47x04 or YES/1=47x16 */
uint8_t bytevalue=255;  /* byte to write to entire SRAM array             */
/*-----*/

/* read and write arrays for I2C, these are passed as a constant pointer to non-constant data
 * each array is large enough for the entire SRAM of a 47x16
 * for the w_array
 * byte 0 addrh
 * byte 1 addrl
 * bytes 2-2049 hold the data
 */
uint8_t R_array[2048]; /* data in 0-2047 */
uint8_t W_array[2050]; /* data in 2-2049 */

qm_gpio_port_config_t GPIOcfg;  /* GPIO configuration structure- see qm_gpio.h */
qm_i2c_config_t I2Ccfg; 	    /* I2C structure variables                     */
qm_i2c_status_t I2Cstatus; 	    /* see i2c.h for values                        */

/*__________________________________________________________________________*/
int main(void){
	int rescode,x;
	uint16_t Haddr,mm;

	QM_PRINTF("EERAM_SET\n");

	/* configure GPIO bit for the HS connection and clear it              */
    /* The HS_PIN is used to trigger auto-store when AM bit=1.            */
	/* This feature is not used here - uncomment next four lines if using */
	/* qm_pmux_select(HS_PIN, QM_PIN_0_FN_GPIO_0); */
	/* GPIOcfg.direction = BIT(HS_PIN);            */
	/* qm_gpio_set_config(QM_GPIO_0, &GPIOcfg);    */
	/* qm_gpio_clear_pin(QM_GPIO_0, HS_PIN);       */
	/* The HS_PIN is used to trigger auto-store when AM bit=1.This feature is not used here	*/

	/* configure I2C and quit if error */
	if ( (rescode=setI2C() )!=0 ) {
		QM_PRINTF("Error: Could not configure I2C (errno=%d)\n",rescode);
		return 1;
	}
	/* read and print the array before the write */
	if((rescode=read_SRAM())!=0){
		return rescode; /* read error */
	}

	/* write the SRAM array to 'byte' */
	if( (rescode=write_SRAM() ) !=0 ){
		return rescode; /* write error */
	}

	/* read and print the array after the write */
	if( (rescode=read_SRAM() )!=0){
		return rescode; /* read error */
	}
	/* compare the read and write arrays */
	if(Hdensity){
		Haddr=2048;
	}
	else{
		Haddr=512;
	}
	mm=0;
	for(x=0;x<Haddr;x++){
		if(R_array[x]!=W_array[x+2]){
			QM_PRINTF("Mismatch at Addr: %d  Read=%d  Write=%d\n",x,R_array[x],W_array[x+2]);
			mm++;
		}
	}
	if(mm==0){
		QM_PRINTF("\n\nRead and Write data match.");
	}
	else{
		QM_PRINTF("\n\nError: %d mismatches in Read and Write data!",mm);
	}
	return 0;
}
/*___________________________________________________________________________*/
int setI2C(void){

	/*  Enable I2C 0 */
	clk_periph_enable(CLK_PERIPH_CLK | CLK_PERIPH_I2C_M0_REGISTER);
	/* set IO pins for SDA and SCL */
	qm_pmux_select(QM_PIN_ID_6, QM_PMUX_FN_2);
	qm_pmux_select(QM_PIN_ID_7, QM_PMUX_FN_2);

	/* Configure I2C */
	I2Ccfg.address_mode = QM_I2C_7_BIT;
	I2Ccfg.mode = QM_I2C_MASTER;
	I2Ccfg.speed = QM_I2C_SPEED_STD;

	/* set the configuration through the structure and return if failure */
	if (qm_i2c_set_config(QM_I2C_0, &I2Ccfg)) {
		QM_PUTS("Error: I2C_0 config\n");
		return (errno);
	} else {
		return (0);
	}
}
/*___________________________________________________________________________*/
int write_SRAM(void){
	int x,rescode;
	uint16_t length;


	QM_PRINTF("\n\nSetting SRAM array to %d..",bytevalue);
	W_array[0]=0;               /* starting address high       */
	W_array[1]=0;               /* starting address low        */

	if(Hdensity){
		length=2048+2;
	}
	else{
		length=512+2;
	}
	/* fill W-array with the bytevalue to write */
	for(x=2;x<length;x++){
		W_array[x]=bytevalue;
	}
	/* write W_array to the SRAM array */
	rescode=qm_i2c_master_write(QM_I2C_0, SRAMcb, W_array, length, true, &I2Cstatus);
	if(rescode) {
		QM_PRINTF(" Error writing EERAM SRAM array (errno=%d)\n",rescode);
	}
	else{
		QM_PRINTF(" SRAM array written\n");
	}
	return rescode;
}
/*__________________________________________________________________________*/
int read_SRAM(){
	int x,rescode;
	uint16_t length;
	uint8_t addset[2];

	QM_PRINTF("\nReading SRAM array...");
	/* first set the address to the start of SRAM with a write (no data)	 */
	addset[0]=0;
	addset[1]=0;
	/* write the array address with NO stop bit */
	length=2;
	rescode=qm_i2c_master_write(QM_I2C_0, SRAMcb, addset, length, false, &I2Cstatus);
	if(rescode) {
		QM_PRINTF(" Error writing EERAM address (errno=%d)\n",rescode);
		return(rescode);
	}

	/* length is the number of bytes to read */
	if(Hdensity){
		length=2048;
	}
	else{
		length=512;
	}
	/* now read the SRAM array into r_array */
	rescode=qm_i2c_master_read(QM_I2C_0, SRAMcb, R_array, length, true, &I2Cstatus);
	if(rescode) {
		QM_PRINTF(" Error reading EERAM SRAM array (errno=%d)\n",rescode);
		return (rescode);
	}
	else{
		QM_PRINTF(" SRAM array read\n");
	}
	/* print the entire SRAM array to the serial monitor */
	for(x=0;x<length;x++){
		//QM_PRINTF("addr=%d value=%d \n",x,R_array[x]);
		QM_PRINTF("%d ",R_array[x]);
	}
	return rescode;
}
/*__________________________________________________________________________*/
